<?
/*
 * $Horde$
 *
 * Copyright 2001 Robert E. Coyle <robertecoyle@hotmail.com>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

include_once 'DB.php';
include_once 'backend.inc';

class TicketBackendSql extends TicketBackend {
    function initialise() {
	global $conf;

	$this->db = &DB::connect($conf['backend']['dsn'], true);

	if (DB::isError($this->db) || DB::isWarning($this->db)) {
	    pre_var_dump($this->db);
	    return false;
	}

	return true;
    }

    function addTicket(&$info) {
	// get our vars
	$requester = WhupsAuth::getUserID();
	$type = $info['type'];
	$state = $info['state'];
	$priority = $info['priority'];
	$module = $info['module'];
	$comment = $info['comment'];

	// get the new unique ids for this ticket and the inital comment
	$new_id = $this->db->nextId('ticket');

	if (DB::isError($new_id))
	    return false;

	$new_comment_id = $this->db->nextId('comment');

	if (DB::isError($new_comment_id))
	    return false;

	// set up our vars
	$comment = addslashes($comment);

	// create the ticket
	$result = $this->_query("INSERT INTO ticket (id, requester, owner, type, state, " .
				"    priority, module, comment, timestamp) " .
				"VALUES ($new_id, $requester, 0, $type, $state," .
				"    $priority, $module, $new_comment_id, NOW())");

	if (DB::isError($result))
	    pre_var_dump($result);

	// create the initial comment
	$result = $this->_query("INSERT INTO comment (id, ticket_id, creator, comment, timestamp) " .
				"VALUES ($new_comment_id, $new_id, $requester, \"$comment\", NOW())");

	if (DB::isError($result))
	    pre_var_dump($result);

	// update state_log (tm)
	$result = $this->_updateStateLog($new_id, "NULL", $state, $requester, $new_comment_id);

	if (DB::isError($result))
	    pre_var_dump($result);

	return array($new_id, $new_comment_id);
    }

    function addComment(&$info) {
	$ticket_id = $info['id'];
	$comment = $info['newcomment'];
	$creator = $info['creator'];

	$result = $this->_addComment($ticket_id, $comment, $creator);

	$this->notifyComment($ticket_id, $comment, $creator);
    }

    function assignTicket(&$info) {
	$ticket_id = $info['id'];
	$email = $info['assignee'];
	$state = $info['tostate'];
	$morecomment = $info['newcomment'];

	$details = $this->getUserDetailsFromEmail($email);
	$assignee = $details['id'];
	$name = $details['name'];

	$comment = "Assigned ticket to $name <$email>";

	if (!empty($morecomment))
	    $comment .= "\n\n$morecomment";

	$details = $this->getTicketDetails($ticket_id);

	$this->_query("UPDATE ticket SET owner = $assignee, state = $state WHERE id = $ticket_id");
	$new_comment_id = $this->_addComment($ticket_id, $comment, WhupsAuth::getUserID());
	$this->_updateStateLog($ticket_id, $details['state'], $state, WhupsAuth::getUserID(), $new_comment_id);

	$this->notifyAssignation($details, $morecomment, $name, $email, $state);
    }

    function setTicketState(&$info) {
	$ticket_id = $info['id'];
	$newcomment = $info['newcomment'];
	$tostate = $info['state'];
	$type = $info['type'];

	$this->_query("UPDATE ticket SET state = $tostate WHERE id = $ticket_id");
	$new_comment_id = $this->_addComment($ticket_id, $newcomment, WhupsAuth::getUserID());
	$this->_updateStateLog($ticket_id, '', $tostate, WhupsAuth::getUserID(), $new_comment_id);

	$this->notifyStateChange($ticket_id, $type, $tostate, $newcomment, WhupsAuth::getUserID());
    }

    function setTicketPriority(&$info) {
	$ticket_id = $info['id'];
	$newcomment = $info['newcomment'];
	$priority = $info['priority'];
	$type = $info['type'];

	$this->_query("UPDATE ticket SET priority = $priority WHERE id = $ticket_id");
	$new_comment_id = $this->_addComment($ticket_id, $newcomment, WhupsAuth::getUserID());

	$this->notifyPriorityChange($ticket_id, $type, $priority, $newcomment, WhupsAuth::getUserID());
    }

    function addUser(&$info) {
	$name = addslashes($info['name']);
	$email = addslashes($info['email']);
	$passwd = addslashes($info['passwd']);

	$result = $this->_getOne("SELECT name FROM user WHERE email = \"$email\"");

	if (!empty($result['name']))
	    return false;

	$new_id = $this->db->nextId('user');
	$result = $this->_query("INSERT INTO user (id, name, email, password) " .
				"VALUES ($new_id, \"$name\", \"$email\", \"$passwd\")");

	if (DB::isError($result))
	    return 0;

	$this->_query("INSERT INTO user_searches (user, name, getvars) " .
		      "VALUES ($new_id, \"My Tickets\", \"id=&module=0&type=0&owner=$new_id\")");
	$this->_query("INSERT INTO user_searches (user, name, getvars) " .
		      "VALUES ($new_id, \"My Placed Tickets\", \"id=&module=0&type=0&requester=$new_id\")");

	return $new_id;
    }

    function &getTicketsByProperties(&$info, $munge = true) {
	// search conditions
	$where = $this->_generateWhere(
	    'ticket',
	    array('id', 'requester', 'owner', 'type', 'state', 'priority', 'module'),
	    $info, 'integer');

	// limits
	$start = $info['start'];
	$count = $info['count'];

	$limit = $this->_constructLimit($start, $count);

	// don't even try it.
	if (empty($where) && empty($limit))
	    return array();

	$fields = array('id', 'requester', 'owner', 'state', 'type', 'priority', 'module', 'comment');
	$fields = $this->_prefixTableToColumns('ticket', $fields);
	$fields .= ', UNIX_TIMESTAMP(ticket.timestamp) AS timestamp';
	$tables = "ticket";

	$nouc = $info['nouc'];
	$nores = $info['nores'];
	$nonew = $info['nonew'];
	$noass = $info['noass'];

	if ($nouc || $nores || $nonew || $noass) {
	    $nouc = $nouc ? ' AND state.category <> "unconfirmed"' : '';
	    $nores = $nores ? ' AND state.category <> "resolved"' : '';
	    $nonew = $nonew ? ' AND state.category <> "new"' : '';
	    $noass = $noass ? ' AND state.category <> "assigned"' : '';

	    $where = $this->_addWhere($where, 1, "(ticket.type = state.type AND ticket.state = state.state$nouc$nores$nonew$noass)");
	    $tables .= ', state';
	    $havestate = 1;
	}

	if ($munge) {
	    $fields = "$fields, " .
		"type.name AS _type_name, " .
		"state.name AS _state_name, " .
		"module.name AS _module_name, " .
		"priority.name AS _priority_name";

	    $tables = "$tables, type, module, priority" . ($havestate ? '' : ', state');

	    $where = $this->_addWhere($where, 1,
		"(ticket.type = type.id) AND " .
		($havestate ? '' : "(ticket.type = state.type AND ticket.state = state.state) AND ") .
		"(ticket.module = module.id) AND " .
		"(ticket.type = priority.type AND ticket.priority = priority.priority)");
	}

	return $this->_getAll(
	    "SELECT $fields FROM $tables WHERE $where ORDER BY ticket.priority, ticket.timestamp$limit");
    }

    function getTicketDetails($ticket) {
	$info = array('id' => $ticket);

	$result = $this->getTicketsByProperties($info);
	$result = $result[0];

	if (!empty($result['requester'])) {
	    $rname = $this->_getAll("SELECT name, email FROM user WHERE id = " . $result['requester']);
	    $result['_requester_name'] = $rname[0]['name'];
	    $result['_requester'] = $rname[0]['email'];
	}

	if (!empty($result['owner'])) {
	    $oname = $this->_getAll("SELECT name, email FROM user WHERE id = " . $result['owner']);
	    $result['_owner_name'] = $oname[0]['name'];
	    $result['_owner'] = $oname[0]['email'];
	}

	return $result;
    }

    function getComments($ticket, $munge = true) {
	// search conditions
	$where = "ticket_id = $ticket";
	$tables = 'comment';

	$fields = array('id', 'ticket_id', 'creator', 'comment');
	$fields = $this->_prefixTableToColumns('comment', $fields);
	$fields .= ', UNIX_TIMESTAMP(timestamp) AS timestamp';

	if ($munge) {
	    $fields .= ', user.name AS _creator_name, user.email AS _creator';
	    $where = $this->_addWhere($where, 1, "(comment.creator = user.id)");
	    $tables .= ', user';
	}

	$results = $this->_getAll("SELECT $fields FROM $tables WHERE $where ORDER BY comment.timestamp");

	if (!$munge) {
	    return $results;
	}

	// TODO: get attachments
	return $results;
    }

    function getModules() {
	return $this->_getAssoc('SELECT id, name FROM module');
    }

    function getTypes() {
	return $this->_getAssoc('SELECT id, name FROM type');
    }

    function getStates($type, $category = '', $notcategory = '') {
	$where = "type = $type";
	$where = $this->_addWhere($where, $category, "category = \"$category\"");
	$where = $this->_addWhere($where, $notcategory, "category <> \"$notcategory\"");

	return $this->_getAssoc("SELECT state, name FROM state WHERE $where");
    }

    function getStateName($type, $state) {
	return $this->_getOne("SELECT name FROM state WHERE type = $type AND state = $state");
    }

    function getPriorities($type) {
	return $this->_getAssoc("SELECT priority, name FROM priority WHERE type = $type");
    }

    function getPriorityName($type, $priority) {
	return $this->_getOne("SELECT name FROM priority WHERE type = $type AND priority = $priority");
    }

    function getUserFromEmail($email) {
	$email = addslashes($email);
	return $this->_getOne("SELECT id FROM user WHERE email = \"$email\"");
    }

    function getUserDetails($id) {
        $result = $this->_getAll("SELECT name, email FROM user WHERE id = $id");
	return $result[0];
    }

    function getUserDetailsFromEmail($email) {
	$email = addslashes($email);
        $result = $this->_getAll("SELECT id, name FROM user WHERE email = \"$email\"");
	return $result[0];
    }

    function getUserSearches($user, $base) {
	$base = addslashes($base) . '?';
	return $this->_getAssoc("SELECT name, CONCAT(\"$base\", getvars) AS url FROM user_searches WHERE user = $user");
    }

    // tricky - handle case where owner = requester
    function getListeners($ticket, $withowner = true, $withrequester = true) {
	$tables = "user, ticket_listeners, ticket";
	$where = "(ticket_listeners.ticket = $ticket AND user.id = ticket_listeners.user)";

	if (!$withowner)
	    $where .= " AND (ticket.id = $ticket AND ticket.owner <> user.id AND ticket.owner <> ticket.requester)";
	else
	    $where .= " OR (ticket.id = $ticket AND ticket.owner = user.id)";

	if (!$withrequester)
	    $where .= " AND (ticket.id = $ticket AND ticket.requester <> user.id)";
	else
	    $where .= " OR (ticket.id = $ticket AND ticket.requester = user.id AND ticket.requester <> ticket.owner)";

	return $this->_getAll("SELECT user.name, user.email FROM $tables WHERE $where GROUP BY user.id");
    }

    // -- private methods --

    function _generateWhere($table, $fields, &$info, $type) {
	$where = "";

	foreach ($fields as $field) {
	    $prop = $info[$field];
	    $success = @settype($prop, $type);
	    $where = $this->_addWhere($where, $prop && $success, "$table.$field = \"" . addslashes($prop) . '"');
	}

	foreach ($fields as $field) {
	    $prop = $info["not$field"];

	    if (strpos($prop, ',') === false) {
		$success = @settype($prop, $type);
		$where = $this->_addWhere($where, $prop && $success, "$table.$field <> \"" . addslashes($prop) . '"');
	    } else {
		$set = explode(',', $prop);

		foreach ($set as $prop) {
		    $success = @settype($prop, $type);
		    $where = $this->_addWhere($where, $prop && $success, "$table.$field <> \"" . addslashes($prop) . '"');
		}
	    }
	}

	return $where;
    }

    function _generateTextSearch($field, $tomatch) {
	$matches = split('[ \\n\\t]+', $tomatch);

	foreach ($matches as $match) {
	    $where = $this->_addWhere($where, 1, "$field LIKE \"% $match %\"", 'OR');
	}

	return $where;
    }

    function _addWhere($where, $condition, $clause, $conjunction = 'AND') {
	if (!empty($condition)) {
	    if (!empty($where))
		$where .= " $conjunction ";

	    $where .= "($clause)";
	}

	return $where;
    }

    function _constructLimit($start, $count) {
	if (empty($count))
	    return;

	$limit = " LIMIT ";

	if (!empty($start))
	    $limit .= "$start, ";

	return $limit . $count;
    }

    function _prefixTableToColumns($table, &$columns) {
	$join = "";

	foreach ($columns as $column) {
	    $clause .= "$join$table.$column";
	    $join = ', ';
	}

	return $clause;
    }

    function _updateStateLog($ticket, $fromstate, $tostate, $owner, $comment_id) {
	if (empty($fromstate))
	    $fromstate = $this->_getOne("SELECT state FROM ticket WHERE id = $ticket");

	return $this->_query("INSERT INTO state_log (ticket_id, from_state, to_state, timestamp, owner, explanation) " .
			     "VALUES ($ticket, $fromstate, $tostate, NOW(), $owner, $comment_id)");
    }

    function _addComment($ticket_id, $comment, $creator) {
	// get the new id
	$new_comment_id = $this->db->nextId('comment');

	if (DB::isError($new_comment_id))
	    return false;

	// set up our vars
	$comment = addslashes($comment);

	// add the row
	$this->_query("INSERT INTO comment (id, ticket_id, creator, comment, timestamp) " .
		      "VALUES ($new_comment_id, $ticket_id, $creator, \"$comment\", NOW())");

	return $new_comment_id;
    }

    // I can't believe PEAR doesn't support this already
    function &_query($query) {
	$start = getmicrotime();
	$result = $this->db->query($query);
	$end = getmicrotime();

	$this->_debugDB($query, $result, $end - $start);
	return $result;
    }

    function &_getAll($query) {
	$start = getmicrotime();
	$result = $this->db->getAll($query, DB_FETCHMODE_ASSOC);
	$end = getmicrotime();

	$this->_debugDB($query, $result, $end - $start);
	return $result;
    }

    function &_getAssoc($query) {
	$start = getmicrotime();
	$result = $this->db->getAssoc($query);
	$end = getmicrotime();

	$this->_debugDB($query, $result, $end - $start);
	return $result;
    }

    function &_getOne($query) {
	$start = getmicrotime();
	$result = $this->db->getOne($query);
	$end = getmicrotime();

	$this->_debugDB($query, $result, $end - $start);
	return $result;
    }

    function _debugDB(&$query, &$result, $time) {
	global $debug_db;

	if ($debug_db) {
	    $time = sprintf("Completed in %.3f seconds.", $time);

	    echo "<span class=\"debug\"><b>$query</b><br /><pre>\n";
	    print_r($result);
	    echo "\n$time\n";
	    echo "</pre></span><br />\n";
	}
    }
}

?>
