<?
/*
 * $Horde$
 *
 * Copyright 2001 Robert E. Coyle <robertecoyle@hotmail.com>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

// ABC for backends

class TicketBackend {
    function initialise() {}
    function addTicket($requester, $owner, $type, $state, $priority, $module) {}
    function &getTicketsByProperties(&$vars) {}

    function notifyComment($ticket_id, $comment, $creator) {
	$details = $this->getUserDetails($creator);
	$cname = $details['name'];
	$cemail = $details['email'];

	$subject = "New comment for ticket $ticket_id";
	$message1 = "$cname <$cemail> added the following comment to ticket $ticket_id:";
	$message1 = $this->_splitMessage($message1);
	$message2 = $this->_splitMessage($comment);

	$message = $message1 . "\n\n" . $message2 . "\n\n";

	$this->_disclaimer($message);
	$this->_sendToAll($ticket_id, $subject, $message);
    }

    function notifyAssignation(&$olddetails, &$comment, $ownername, $owneremail, $newstate) {
	$mydetails = $this->getUserDetails(WhupsAuth::getUserID());

	$myname = $mydetails['name'];
	$myemail = $mydetails['email'];

	$ticket_id = $olddetails['id'];
	$state_name = $this->getStateName($olddetails['type'], $newstate);

	$message2 = empty($comment) ? "\n\n" : ("\n\n" . $this->_splitMessage($comment) . "\n\n");

	$subject = "Ticket $ticket_id has been assigned";
	$message = "$myname <$myemail> has assigned ticket $ticket_id to $ownername <$owneremail>. ";
	$message .= "It is now in the $state_name state.";
	$message = $this->_splitMessage($message);
	$message .= $message2;

	$this->_disclaimer($message);
	$this->_sendToAllButOwner($ticket_id, $subject, $message);

	$subject = "You have been assigned ticket $ticket_id";
	$message = "$myname <$myemail> has assigned ticket $ticket_id to you.\n";
	$message .= "It is now in the $state_name state.";
	$message .= $message2;

	$this->_disclaimer($message);
	$this->_mail("$ownername <$owneremail>", $subject, $message);
    }

    function notifyStateChange($ticket_id, $type, $tostate, $comment, $userid) {
	$mydetails = $this->getUserDetails($userid);
	$statename = $this->getStateName($type, $tostate);

	$myname = $mydetails['name'];
	$myemail = $mydetails['email'];

	$subject = "Ticket $ticket_id has changed state";
	$message = "$myname <$myemail> has changed the state of ticket $ticket_id to state $statename. ";
	$message .= "The explanation was:";
	$message = $this->_splitMessage($message);
	$message .= "\n\n";

	$message .= $this->_splitMessage($comment);
	$message .= "\n\n";

	$this->_disclaimer($message);
	$this->_sendToAll($ticket_id, $subject, $message);
    }

    function notifyPriorityChange($ticket_id, $type, $priority, $comment, $userid) {
	$mydetails = $this->getUserDetails($userid);
	$priorityname = $this->getPriorityName($type, $tostate);

	$myname = $mydetails['name'];
	$myemail = $mydetails['email'];

	$subject = "Ticket $ticket_id has changed priority";
	$message = "$myname <$myemail> has changed the priority of ticket $ticket_id to $priorityname. ";
	$message .= "The explanation was:";
	$message = $this->_splitMessage($message);
	$message .= "\n\n";

	$message .= $this->_splitMessage($comment);
	$message .= "\n\n";

	$this->_disclaimer($message);
	$this->_sendToAll($ticket_id, $subject, $message);
    }

    function _disclaimer(&$message) {
	$message1 = "This mail was automatically generated by the Whups request tracking ";
	$message1 .= "system (http://horde.org/whups/).";

	$message2 = "If you have recieved this mail in error, please contact ";
	$message2 .= getContactDetails() . ".  Please do not ";
	$message2 .= "contact the Whups development team, who almost certainly do not ";
	$message2 .= "have anything to do with this email.";

	$message .= "-- \n";
	$message .= $this->_splitMessage($message1);
	$message .= "\n\n";
	$message .= $this->_splitMessage($message2);
	$message .= "\n";
    }

    function _sendToAllButOwner($ticket_id, $subject, $message) {
	$addresses = $this->getListeners($ticket_id, false);

	foreach ($addresses as $key => $info) {
	    $name = $info['name'];
	    $email = $info['email'];

	    $this->_mail("$name <$email>", $subject, $message);
	}
    }

    function _sendToAll($ticket_id, $subject, $message) {
	$addresses = $this->getListeners($ticket_id);

	foreach ($addresses as $key => $info) {
	    $name = $info['name'];
	    $email = $info['email'];

	    $this->_mail("$name <$email>", $subject, $message);
	}
    }

    function _mail($to, $subject, $message) {
	$subject = "[Whups] $subject";

	echo "<font color=\"white\"><pre>\n";
	echo "Sending mail to " . htmlspecialchars($to) . "\n";
	echo "Subject: " . htmlspecialchars($subject) . "\n\n";
	echo htmlspecialchars($message) . "\n";
	echo "</pre></font>\n";
    }

    function _splitMessage($message) {
	$set = split("[ \t]+", $message);
	$len = 0;

	foreach ($set as $substr) {
	    while (($pos = strpos($substr, "\n")) !== false) {
		$result .= substr($substr, 0, $pos);
		$len = 0;
		$substr = substr($substr, $pos + 1);
	    }

	    $slen = strlen($substr);

	    if (!$slen)
		continue;

	    $len += $slen + 1;

	    if ($len > 80) {
		$len = $slen;
		$result .= "\n" . $substr . ' ';
	    } else {
		$result .= $substr . ' ';
	    }
	}

	return $result;
    }
}

?>
