<?
/*
 * $Horde$
 *
 * Copyright 2001 Robert E. Coyle <robertecoyle@hotmail.com>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

include_once 'form.inc';

class SearchForm extends Form {
    function SearchForm(&$vars, &$backend) {
	$this->Form();

	$modules = $backend->getModules();
	$modules = array_merge(array('0' => 'Any'), $modules);
	$modtype = new EnumType($modules);

	$types = $backend->getTypes();
	$types = array_merge(array('0' => 'Any'), $types);
	$typetype = new EnumType($types);

	$var1 = new Variable('Ticket ID', 'id', getIntType(), false);
	$var2 = new Variable('Module Name', 'module', $modtype, false);
	$var3 = new Variable('Ticket Type', 'type', $typetype, false);
	$var4 = new Variable('Count', 'count', getIntType(), false, true, true);

	$var4->setDefault(10);

	$this->addVariable($var1);
	$this->addVariable($var2);
	$this->addVariable($var3);
	$this->addVariable($var4);
    }
}

class SearchNarrow1Form extends Form {
    function SearchNarrow1Form(&$vars, &$backend) {
	$this->Form();

	$type = $vars->getVar('type');

	$states = $backend->getStates($type);
	$states = array_merge(array('0' => 'Any'), $states);
	$states = new EnumType($states);

	$priorities = $backend->getPriorities($type);
	$priorities = array_merge(array('0' => 'Any'), $priorities);
	$priorities = new EnumType($priorities);

	$var1 = new Variable('Ticket State', 'state', $states, false);
	$var2 = new Variable('Priority', 'priority', $priorities, false);
	$var3 = new Variable('Hide Unconfirmed', 'nouc', getBooleanType(), false);
	$var4 = new Variable('Hide New', 'nonew', getBooleanType(), false);
	$var5 = new Variable('Hide Assigned', 'noass', getBooleanType(), false);
	$var6 = new Variable('Hide Resolved', 'nores', getBooleanType(), false);

	$this->addVariable($var1);
	$this->addVariable($var2);
	$this->addVariable($var3);
	$this->addVariable($var4);
	$this->addVariable($var5);
	$this->addVariable($var6);
    }
}

class TicketDetailsForm extends Form {
    function TicketDetailsForm(&$vars, &$backend) {
	$this->Form('ticketdetailsform');

	$var1 = new Variable('Module Name', '_module_name', getTextType(), true, true);
	$var2 = new Variable('Ticket Type', '_type_name', getTextType(), true, true);
	$var3 = new Variable('Ticket State', '_state_name', getTextType(), true, true);
	$var4 = new Variable('Priority', '_priority_name', getTextType(), true, true);
	$var5 = new Variable('Owner', '_owner', getEmailType(), true, true);
	$var6 = new Variable('Requester', '_requester', getEmailType(), true, true);

	$var5->setDefault("Unassigned");

	$this->addVariable($var1);
	$this->addVariable($var2);
	$this->addVariable($var3);
	$this->addVariable($var4);
	$this->addVariable($var5);
	$this->addVariable($var6);
    }
}

class AddCommentForm extends Form {
    function AddCommentForm(&$vars, &$backend) {
	$this->Form('addcommentform');

	$var1 = new Variable('Ticket ID', 'id', getIntType(), true, true, true);
	$var2 = new Variable('Comment', 'newcomment', getLongTextType(), true);

	$this->addVariable($var1);
	$this->addVariable($var2);
    }
}

class AssignTicketStep1Form extends Form {
    function AssignTicketStep1Form(&$vars, &$backend) {
	$this->Form('assignticketstep1form');

	$var1 = new Variable('Ticket Type', 'type', getIntType(), true, true, true);
	$var2 = new Variable('Ticket ID', 'id', getIntType(), true, true, true);
	$var3 = new Variable('Assignee', 'assignee', getUserType($backend), true);

	$this->addVariable($var1);
	$this->addVariable($var2);
	$this->addVariable($var3);
    }
}

class AssignTicketStep2Form extends Form {
    function AssignTicketStep2Form(&$vars, &$backend) {
	$this->Form('assignticketstep2form');

	$type = $vars->getVar('type');
	$states = $backend->getStates($type, 'assigned');
	$states = new EnumType($states);

	$var1 = new Variable('New State', 'tostate', $states, true);
	$var2 = new Variable('Additional Comment', 'newcomment', getLongTextType(), false);

	$this->addVariable($var1);
	$this->addVariable($var2);
    }
}

class SetStateStep1Form extends Form {
    function SetStateStep1Form(&$vars, &$backend) {
	$this->Form('setstatestep1form');

	$type = $vars->getVar('type');
	$states = $backend->getStates($type, '', 'unconfirmed');
	$states = new EnumType($states);

	$var1 = new Variable('Ticket ID', 'id', getIntType(), true, true, true);
	$var2 = new Variable('New State', 'state', $states, true);
	$var3 = new Variable('Type', 'type', getIntType(), true, true, true);

	$this->addVariable($var1);
	$this->addVariable($var2);
	$this->addVariable($var3);
    }
}

class SetStateStep2Form extends Form {
    function SetStateStep2Form(&$vars, &$backend) {
	$this->Form('setstatestep2form');
	$var1 = new Variable('Comment', 'newcomment', getLongTextType(), true);
	$this->addVariable($var1);
    }
}

class SetPriorityStep1Form extends Form {
    function SetPriorityStep1Form(&$vars, &$backend) {
	$this->Form('setprioritystep1form');

	$type = $vars->getVar('type');
	$priorities = $backend->getPriorities($type);
	$priorities = new EnumType($priorities);

	$var1 = new Variable('Ticket ID', 'id', getIntType(), true, true, true);
	$var2 = new Variable('New Priority', 'priority', $priorities, true);
	$var3 = new Variable('Type', 'type', getIntType(), true, true, true);

	$this->addVariable($var1);
	$this->addVariable($var2);
	$this->addVariable($var3);
    }
}

class SetPriorityStep2Form extends Form {
    function SetPriorityStep2Form(&$vars, &$backend) {
	$this->Form('setprioritystep2form');
	$var1 = new Variable('Comment', 'newcomment', getLongTextType(), true);
	$this->addVariable($var1);
    }
}

?>
