<?
/*
 * $Horde$
 *
 * Copyright 2001 Robert E. Coyle <robertecoyle@hotmail.com>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

class WhupsVars {
    function WhupsVars(&$vars, &$parent) {
	$this->vars   = $vars;
	$this->parent = $parent;
	$this->prefix = array('');
    }

    function unsetVar($varname) {
	$this->_unsetVar($this->prefix[0] . $varname);
    }

    function getVar($varname) {
	$result = $this->_getVar($this->prefix[0] . $varname, $ignore);
	return $result;
    }

    function getVarWasset($varname, &$wasset) {
	$result = $this->_getVar($this->prefix[0] . $varname, $wasset);
	return $result;
    }

    function getPrefix() {
	return $this->prefix[0];
    }

    function pushPrefix($prefix) {
	array_unshift($this->prefix, $prefix);
    }

    function popPrefix() {
	array_shift($this->prefix);
    }

    function preserveByGet($baseurl) {
	$this->_preserveByGet($baseurl, $this->prefix[0]);
    }

    function preserveByPost() {
	$this->_preserveByPost($this->prefix[0]);
    }

    function preserveVarByPost($varname) {
	$varname = $this->prefix[0] . $varname;
	$value   = $this->_getVar($varname, $wasset);

	if ($wasset)
	    preserveVarByPost($varname, $value);
    }

    // -- private functions --

    function _unsetVar($varname) {
	if (is_object($this->parent))
	    $this->parent->_unsetVar($varname);

	unset($this->vars[$varname]);
    }

    function _getVar($varname, &$wasset) {
	if (is_object($this->parent)) {
	    $var = $this->parent->_getVar($varname, $wasset);

	    if ($wasset)
		return $var;
	}

	$wasset = isset($this->vars[$varname]);
	return $this->vars[$varname];
    }

    function _preserveByGet($baseurl, $prefix) {
	if ($this->parent) 
	    $baseurl = $this->parent->_preserveByGet($baseurl, $prefix);

	if (strchr($baseurl, '?'))
	    $join = '&';
	else
	    $join = '?';

	$prefixlen = strlen($prefix);

	foreach ($this->vars as $name => $value) {
	    if (substr($name, 0, $prefixlen) == $prefix) {
		$baseurl .= $join . urlencode($name) . '=' . urlencode($value);
		$join = '&';
	    }
	}

	return $baseurl;
    }

    function _preserveByPost($prefix) {
	if ($this->parent)
	    $this->parent->_preserveByPost($prefix);

	$prefixlen = strlen($prefix);

	foreach ($this->vars as $name => $value) {
	    if (substr($name, 0, $prefixlen) == $prefix) {
		preserveVarByPost($name, $value);
	    }
	}
    }
}

function &getDefaultVars($override = "") {
    global $HTTP_GET_VARS;
    global $HTTP_POST_VARS;

    return new WhupsVars(
	$HTTP_POST_VARS,
	new WhupsVars(
	    $HTTP_GET_VARS, $override));
}

?>
